; This program is free software. It comes without any warranty, to
; the extent permitted by applicable law. You can redistribute it
; and/or modify it under the terms of the Do What The Fuck You Want
; To Public License, Version 2, as published by Sam Hocevar. See
; http://sam.zoy.org/wtfpl/COPYING for more details.


;====== Math Routines ==========================================================
;------ Milos Bazelides routines -----------------------------------------------
; These routines are by Milos "baze" Bazelides, baze_at_baze_au_com
; Retrived from http://baze.au.com/misc/z80bits.html 25 October 2008
#ifdef NEVER
MultHByE:
; Standard optimized 8-bit multiply.  Kept in RAM for best speed.
; Input: H = Multiplier, E = Multiplicand
; Output: HL = Product
	xor	a
	ld	l, a
	ld	d, a
	sla	h		; optimised 1st iteration
	jr	nc,$+3
	ld	l,e
	;
	ld	b, 7
multHbyEloop:
	add	hl,hl		; unroll 7 times
	jr	nc,$+3		; ...
	add	hl,de		; ...
	djnz	multHbyEloop
	ret
#endif


;------ ------------------------------------------------------------------------
DivACByDE:
; Divides A:C by DE
; Inputs:
;  - A:C: Dividend
;  - DE: Divisor
; Outputs:
;  - A:C: Quotient
;  - HL: Remainder
; Destroys:
;  - AF
;  - BC
	ld	hl, 0
	ld	b, 16
_:	sll	c
	rla
	adc	hl, hl
	sbc	hl, de
	jr	nc, $ + 4
	add	hl, de
	dec	c
	djnz	-_
	ret


;------ ------------------------------------------------------------------------
#ifdef NEVER
DispHL:
; Input:
;  - HL
; Output:
;  - Word displayed
; Destroys:
;  - AF
	ld	a, h
	call	DispByte
	ld	a, l
DispByte:
; Input:
;  - A: Byte
; Output:
;  - Byte displayed
; Destroys:
;  - AF
	push	af
	rra
	rra
	rra
	rra
	call	+_
	pop	af
_:	or	0F0h
	daa
	add	a, 0A0h
	adc	a, 40h
	call	PutC
;	b_call(_PutC)
	ret
#endif


;------ ------------------------------------------------------------------------
#ifdef NEVER
DispDecimal:
; Input:
;  - HL
; Output:
;  - 16-bit number displayed in decimal
; Destroys:
;  - BC
;  - AF
	ld	bc,-10000
	call	+_
	ld	bc,-1000
	call	+_
	ld	bc,-100
	call	+_
	ld	c,-10
	call	+_
	ld	c,b
_:	ld	a,'0'-1
_:	inc	a
	add	hl,bc
	jr	c, -_
	sbc	hl,bc
	call	PutC
	ret
#endif

	
;------ ------------------------------------------------------------------------
Random16:
; Inputs:
;  - none
; Output:
;  - HL: 16-bit number, period 65536
; Destroys:
;  - AF
;  - DE
; "This generator . . . was taken from an old ZX Spectrum game and slightly
;  optimised."
	ld	de, (randomVal)		; Seed is usually 0
	ld	a, d
	ld	h, e
	ld	l, 253
	or	a
	sbc	hl, de
	sbc	a, 0
	sbc	hl, de
	ld	d, 0
	sbc	a, d
	ld	e, a
	sbc	hl, de
	jr	nc, +_
	inc	hl
_:	ld	(randomVal), hl
	ret


;====== Routines ===============================================================

;------ ------------------------------------------------------------------------
ClearMem:
; Clears a section of memory SLOW! :p
; Input:
;  - HL: Location to kill
;  - B: Number of bytes to kill
;  - A: What to clear it with
	ld	(hl), a
	inc	hl
	djnz	ClearMem
	ret


#ifdef	NEVER
|  The random number generator routine was provided   | 
|  by Zeda Thomas.                                    | 
;------ Rand24 -----------------------------------------------------------------
; From Zeda's library
; Inputs:
;  - seed1, seed2
; Outputs:
;  - HLA is the pseudo-random number
;  - seed1, seed2 incremented accordingly
; Destroys:
;  - BC
;  - DE
; Notes:
; seed1*243+83 mod 65519 -> seed1
; seed2*251+43 mod 65521 -> seed2
; Output seed1*seed2
	ld	hl, (seed1)
	xor	a
	ld	b, h		\ ld	c, l
	ld	de, 83
	add	hl, hl		\ rla			;2
	add	hl, bc		\ adc	a, d		;3
	add	hl, hl		\ rla			;6
	add	hl, bc		\ adc	a, d		;7
	add	hl, hl		\ rla			;14
	add	hl, bc		\ adc	a, d		;15
	add	hl, hl		\ rla			;30
	add	hl, hl		\ rla			;60
	add	hl, hl		\ rla			;120
	add	hl, bc		\ adc	a, d		;121
	add	hl, hl		\ rla			;242
	add	hl, bc		\ adc	a, d		;243
	add	hl, de		\ adc	a, d		;243x+83
; now AHL mod 65519
; Essentially, we can at least subtract A*65519=A(65536-17), so add A*17 to HL
	ex	de, hl
	ld	l, a
	ld	b, h
	ld	c, l
	add	hl, hl
	add	hl, hl
	add	hl, hl
	add	hl, hl
	add	hl, bc
	add	hl, de
	ld	c, 17
	jr	nc, $ + 3
	add	hl, bc
	add	hl, bc
	jr	c, $ + 4
	sbc	hl, bc
	ld	(seed1), hl
; seed1 done, now we need to do seed2
	ld	hl, (seed2)
; seed1*243+83 mod 65519 -> seed1
; seed2*251+43 mod 65521 -> seed2
; Output seed1*seed2
	xor	a
	ld	b, h		\ ld	c, l
	ld	de, 43
	add	hl, hl		\ rla			;2
	add	hl, bc		\ adc	a, d		;3
	add	hl, hl		\ rla			;6
	add	hl, bc		\ adc	a, d		;7
	add	hl, hl		\ rla			;14
	add	hl, bc		\ adc	a, d		;15
	add	hl, hl		\ rla			;30
	add	hl, bc		\ adc	a, d		;31
	add	hl, hl		\ rla			;62
	add	hl, hl		\ rla			;124
	add	hl, bc		\ adc	a, d		;125
	add	hl, hl		\ rla			;250
	add	hl, bc		\ adc	a, d		;251
	add	hl, de		\ adc	a, d		;251x+83
; now AHL mod 65521
; Essentially, we can at least subtract A*65521=A(65536-15), so add A*15 to HL
	ex	de, hl
	ld	l, a
	ld	b, h
	ld	c, l
	add	hl, hl
	add	hl, hl
	add	hl, hl
	add	hl, hl
	sbc	hl, bc
	add	hl, de
	ld	c, 15
	jr	nc, $ + 3
	add	hl, bc
	add	hl, bc
	jr	c, $ + 4
	sbc	hl, bc
	ld	(seed2), hl
; now seed1 and seed 2 are computed
	ld	bc, (seed1)
	ex	de, hl
	call	BC_Times_DE
	ex	de, hl
	ld	l, b
	ld	h, 0
	ld	b, h
	ld	c, l
	add	hl, hl
	add	hl, hl
	add	hl, bc
	add	hl, hl
	add	hl, hl
	add	hl, bc
	add	hl, hl
	add	hl, bc
	ld	c, d
	ld	d, e
	ld	e, a
	ld	a, c
	sbc	hl, bc
	sbc	a, b
	ret	nc
	ld	c, 43
	add	hl, bc
	ret
BC_Times_DE:
; BHLA is the result
	ld	a, b
	or	a
	ld	hl, 0
	ld	b, h
; 1
	add	a, a
	jr	nc, $ + 4
	ld	h, d
	ld	l, e
; 2
	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, b
; 227+10b-7p
	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, b

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, b

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, b

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, b

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, b

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, b

; ===
; AHL is the result of B*DE*256
	push	hl
	ld	h, b
	ld	l, b
	ld	b, a
	ld	a, c
	ld	c, h
; 1
	add	a, a
	jr	nc, $ + 4
	ld	h, d
	ld	l, e
; 2
	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, c
; 227+10b-7p
	add	hl,hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, c

	add	hl ,hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, c

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, c

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, c

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, c

	add	hl, hl
	rla
	jr	nc, $ + 4
	add	hl, de
	adc	a, c

	pop	de
; Now BDE*256+AHL
	ld	c, a
	ld	a, l
	ld	l, h
	ld	h, c
	add	hl, de
	ret	nc
	inc	b
; BHLA is the 32-bit result
	ret
Rand24_end:
#endif